#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import os

extensions = { }

def registerExtensions (ext_list, arch_type):
	global extensions
	for e in ext_list:
		extensions[e] = arch_type

# Try register zip file handler
try:
	from archtypes import ziphandler
	registerExtensions (ziphandler.getExtensions (), ziphandler.ZipHandler)
except ImportError:
	pass

class Archiver:

	def __init__ (self, parent):
		if __debug__:
			print "Creating archiver object."

		self.parent = parent

		# Grap the global module copy
		global extensions
		self.extensions = extensions
		if __debug__:
			print "Archiver has %s extensions registered." %len (self.extensions)

	def findSupportedHandler (self, path):
		filename = os.path.basename (path).lower ()
		ext = None
		for i in range (len (filename)):
			if filename[i] == '.':
				if self.extensions.has_key (filename[i:]):
					ext = filename[i:]
					break
		return ext

	def getExtHandler (self, ext):
		try:
			return self.extensions[ext]
		except KeyError:
			return None

	def getStatus (self, path):
		ext = self.findSupportedHandler (path)
		handler = self.getExtHandler (ext)
		if handler:
			try:
				inst = handler (path=path)
				if inst.isValid ():
					return _("Ok")
				else:
					return _("Invalid")
			except:
				return _("Error")
		else:
			return _("Unknown")

	def getType (self, path):
		ext = self.findSupportedHandler (path)
		handler = self.getExtHandler (ext)
		if handler:
			try:
				inst = handler (path=path)
				return inst.getDescription ()
			except:
				pass
		return _("Unknown")

	def unpack (self, path, loc):
		ext = self.findSupportedHandler (path)
		handler = self.getExtHandler (ext)
		if handler:
			try:
				inst = handler (path=path, parent=self.parent, intl=_)
				return inst.unpack (loc)
			except Exception, strerror:
				raise Exception, _(strerror)
		return None
