#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import archhandler
import zipfile

import os
import struct
from wxPython.wx import *

def getExtensions ():
	return [ '.zip' ]

class ZipHandler (archhandler.ArchHandler):

	def __init__ (self, **kwargs):
		archhandler.ArchHandler.__init__ (self, **kwargs)

	def isValid (self):
		path = self.getPath ()
		if isinstance (path, str):
			if zipfile.is_zipfile (path):
				zip = zipfile.ZipFile (path, 'r')
				bad_file = zip.testzip ()
				if bad_file is None:
					return True
		return False

	def getDescription (self):
		return "ZIP"

	def unpack (self, loc):
		if not os.path.exists (loc):
			try:
				os.mkdir (loc)
			except OSError, strerror:
				raise

		if not os.path.isdir (loc):
			raise ValueError, loc

		zip_file = None
		progress = None
		file_list = [ ]
		try:
			try:
				zip_file = zipfile.ZipFile (self.path, 'r')
				names = zip_file.namelist ()
				progress = wxProgressDialog (
					self.getIntl ("Ftpcube - Unarchiving"),
					self.getIntl ("Unzipping %(file)s") %{ 'file' : os.path.basename (self.path) },
					len (names),
					self.getParent (),
					style=wxPD_APP_MODAL | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME | wxPD_CAN_ABORT)
				for i in range (len (names)):
					name = names[i]
					info = zip_file.getinfo (name)
					writeable = os.path.join (loc, name)
					if writeable[-1] in '/\\':
						os.mkdir (writeable)
					else:
						file = open (writeable, 'w')
						try:
							bytes = zip_file.read (name)
							file.write (bytes)
						finally:
							file.close ()
						file_list.append (writeable)
					# Update every 8th file
					if i % 8 == 0:
						if not progress.Update (i):
							break
			except Exception, strerror:
				raise
		finally:
			if zip_file:
				zip_file.close ()
			if progress:
				progress.Destroy ()

		return file_list
