#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import events

from wxPython.wx import *

class ConsoleWindow (wxTextCtrl):

	idPOPUP_CLEAR = wxNewId ()
	VERSION = "0.4.3"

	def __init__ (self, parent):
		if __debug__:
			print "Making console window."

		wxTextCtrl.__init__ (self, parent, -1, style=wxTE_MULTILINE | wxTE_READONLY |
		                                             wxNO_BORDER | wxTE_RICH2)

		self.format = {
			'local'  : _("LOCAL > %s\n"),
			'remote' : _("REMOTE> %s\n"),
			'error'  : _("! - %s\n")
		}

		self.colors = {
			'local' : None,
			'remote' : None,
			'error' : None
		}

		self.wxColors = {
			'local' : None,
			'remote' : None,
			'error' : None
		}

		# Set ourselves to receive all console events
		events.EVT_CONSOLE (self, self.onConsoleEvent)
		main.getApp ().registerEventListener (events.wxEVT_CONSOLE, self)

		# Set up the console menu handlers
		EVT_RIGHT_DOWN (self, self.onRightClick)

	def displayBanner (self):
		self.insertColorText ('local', _("Welcome to FtpCube v%(ver)s")
		                               %{ 'ver' : self.VERSION })
		self.insertColorText ('local', _("Copyright (C) 2002  Michael Gilfix"))
		self.insertColorText ('local', '==========================================')
		self.insertColorText ('local', '')

	def insertColorText (self, kind, text):
		color = self.getColor (kind)
		text = self.format[kind] % text
		self.SetDefaultStyle (wxTextAttr (color))
		self.AppendText (text)

	def getColor (self, kind):
		current_color = main.getApp ().getConfig ()[kind + '_color']
		if self.colors[kind] == current_color:
			wx_color = self.wxColors[kind]
		else:
			wx_color = apply (wxColour, current_color)
			self.colors[kind] = current_color
			self.wxColors[kind] = wx_color
		return wx_color

	def onConsoleEvent (self, event):
		self.insertColorText (event.kind, event.msg)

	def onRightClick (self, event):
		menu = self.makePopupMenu ()
		self.PopupMenu (menu, wxPoint (event.m_x, event.m_y))

	def makePopupMenu (self):
		# Create the popup menu
		menu = wxMenu ()
		menu.Append (self.idPOPUP_CLEAR, _("Clear Console"))

		# Set the menu handlers
		EVT_MENU (self, self.idPOPUP_CLEAR, self.onClear)

		return menu

	def onClear (self, event):
		if __debug__:
			print "Clearing console window."
		self.Clear ()
