#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import main
import consolewin
import connectwin
import ctrlwin
import browsewin
import bookmarkwin
import optionwin
import aboutwin
import messages
import events
import protocol
import util
import icons.connect
import icons.quick_connect
import icons.disconnect
import icons.abort
import icons.refresh
import icons.bookmark
import icons.exit

import os.path
from wxPython.wx import *
from wxPython.lib.floatbar import *

class MainWindow (wxFrame):

	# File Menu ID's
	idCONNECT      = wxNewId ()
	idQUICKCONNECT = wxNewId ()
	idBOOKMARKS    = wxNewId ()
	idDISCONNECT   = wxNewId ()
	idCONSOLE      = wxNewId ()
	idOPTIONS      = wxNewId ()
	idEXIT         = wxNewId ()

	# Local menu ID's
	idLOCALUPLOAD = wxNewId ()
	idLOCALRENAME = wxNewId ()
	idLOCALDELETE = wxNewId ()
	idLOCALCREATE = wxNewId ()
	idLOCALCHMOD  = wxNewId ()
	idLOCALCHANGE = wxNewId ()

	# Remote menu ID's
	idREMOTEDOWNLOAD = wxNewId ()
	idREMOTERENAME   = wxNewId ()
	idREMOTEDELETE   = wxNewId ()
	idREMOTECREATE   = wxNewId ()
	idREMOTECHMOD    = wxNewId ()
	idREMOTEREMOVE   = wxNewId ()
	idREMOTECHANGE   = wxNewId ()

	# Help Menu ID's
	idABOUT = wxNewId ()

	# Tool ID's
	idTOOL_CONNECT      = wxNewId ()
	idTOOL_QUICKCONNECT = wxNewId ()
	idTOOL_DISCONNECT   = wxNewId ()
	idTOOL_ABORT        = wxNewId ()
	idTOOL_REFRESH      = wxNewId ()
	idTOOL_BOOKMARK     = wxNewId ()
	idTOOL_QUIT         = wxNewId ()
	idTOOL_ENTRYBOX     = wxNewId ()
	idTOOL_ASCII        = wxNewId ()
	idTOOL_BINARY       = wxNewId ()

	def __init__ (self, parent, title):
		if __debug__:
			print "Making main application window."

		wxFrame.__init__ (self, parent, -1, title, wxDefaultPosition)
		self.SetIcon (main.getApp ().getAppIcon ())

		config = main.getApp ().getConfig ()
		try:
			width, height = config['win_size']
		except:
			width, height = None, None
		if width and height:
			self.SetSize ((width, height))
		else:
			width, height = wxDisplaySize ()
			self.SetSize ((int ( float (width) * 0.75), \
			               int ( float (height) * 0.75)))

		self.makeMenu ()
		toolbar = self.makeToolbar ()

		splitter = wxSplitterWindow (self, -1, style=wxSP_3DSASH)
		self.consolewin = consolewin.ConsoleWindow (splitter)
		csplitter = wxSplitterWindow (splitter, -1, style=wxSP_3DSASH)
		# Set the default control window size
		splitter.SplitHorizontally (self.consolewin, csplitter, 150)

		self.ctrlwin = ctrlwin.ControlWindow (csplitter)
		bsplitter = wxSplitterWindow (csplitter, -1, style=wxSP_3DSASH)
		# Set the default control window size
		csplitter.SplitVertically (self.ctrlwin, bsplitter, 275)

		self.localwin = browsewin.LocalWindow (bsplitter)
		self.remotewin = browsewin.RemoteWindow (bsplitter)
		# Now set the browse splitter position accordingly
		size = (width - 275) / 2
		bsplitter.SplitVertically (self.localwin, self.remotewin, size)

		sizer = wxBoxSizer (wxVERTICAL)
		sizer.Add (splitter, 1, wxEXPAND)
		self.SetAutoLayout (true)
		self.SetSizer (sizer)

		EVT_CLOSE (self, self.onQuit)

		events.EVT_DIRCHANGE (self, self.onDirectoryChange)
		main.getApp ().registerEventListener (events.wxEVT_DIRCHANGE, self)

		self.consolewin.displayBanner ()

	def makeMenu (self):
		self.menu_bar = wxMenuBar ()
		self.menu_bar.Append (self.createFileMenu (), _("&File"))
		self.menu_bar.Append (self.createLocalMenu (), _("&Local"))
		self.menu_bar.Append (self.createRemoteMenu (), _("&Remote"))
		self.menu_bar.Append (self.createHelpMenu (), _("&Help"))

		self.SetMenuBar (self.menu_bar)

	def createFileMenu (self):
		# Create the file menu
		file_menu = wxMenu ()
		file_menu.Append (self.idCONNECT, _("&Connect"), _("Connect to server"))
		file_menu.Append (self.idQUICKCONNECT, _("&Quick Connect"),
		                                       _("Quick connect to server"))
		file_menu.Append (self.idBOOKMARKS, _("&Bookmarks"),
		                                    _("Display the bookmark window"))
		file_menu.Append (self.idDISCONNECT, _("&Disconnect"),
		                                     _("Disconnect from server"))
		file_menu.AppendSeparator ()
		file_menu.Append (self.idOPTIONS, _("&Options"),
		                                  _("Ftpcube options window"))
		file_menu.AppendSeparator ()
		file_menu.Append (self.idCONSOLE, _("C&lear Console"),
		                                  _("Clear console window"))
		file_menu.AppendSeparator ()
		file_menu.Append (self.idEXIT, _("E&xit"), _("Exit Ftpcube"))

		# Set up the event handlers
		EVT_MENU (self, self.idCONNECT, self.onConnect)
		EVT_MENU (self, self.idQUICKCONNECT, self.onQuickConnect)
		EVT_MENU (self, self.idBOOKMARKS, self.onBookmarks)
		EVT_MENU (self, self.idDISCONNECT, self.onDisconnect)
		EVT_MENU (self, self.idCONSOLE, self.onClearConsole)
		EVT_MENU (self, self.idOPTIONS, self.onOptions)
		EVT_MENU (self, self.idEXIT, self.onQuit)

		return file_menu

	def createLocalMenu (self):
		# Create the local menu
		local_menu = wxMenu ()
		local_menu.Append (self.idLOCALUPLOAD, _("&Upload Files"),
		                                       _("Upload files to server"))
		local_menu.Append (self.idLOCALRENAME, _("&Rename Files"),
		                                       _("Rename local files"))
		local_menu.Append (self.idLOCALDELETE, _("&Delete Files"),
		                                       _("Delete local files"))
		local_menu.AppendSeparator ()
		local_menu.Append (self.idLOCALCREATE, _("&Create Directory"),
		                                       _("Create local directory"))
		local_menu.Append (self.idLOCALCHMOD, _("Change &Permisions"),
		                                      _("Change file permissions"))
		local_menu.Append (self.idLOCALCHANGE, _("C&hange Directory"),
		                                       _("Change local directory"))

		# Set up the event handlers
		EVT_MENU (self, self.idLOCALUPLOAD, self.onLocalUpload)
		EVT_MENU (self, self.idLOCALRENAME, self.onLocalRename)
		EVT_MENU (self, self.idLOCALDELETE, self.onLocalDelete)
		EVT_MENU (self, self.idLOCALCREATE, self.onLocalCreate)
		EVT_MENU (self, self.idLOCALCHMOD, self.onLocalChmod)
		EVT_MENU (self, self.idLOCALCHANGE, self.onLocalChange)

		return local_menu

	def createRemoteMenu (self):
		# Create the remote menu
		remote_menu = wxMenu ()
		remote_menu.Append (self.idREMOTEDOWNLOAD, _("&Download Files"),
		                                           _("Download files from server"))
		remote_menu.Append (self.idREMOTERENAME, _("&Rename Files"),
		                                         _("Rename files on server"))
		remote_menu.Append (self.idREMOTEDELETE, _("&Delete Files"),
		                                         _("Delete files from server"))
		remote_menu.AppendSeparator ()
		remote_menu.Append (self.idREMOTECREATE, _("&Create Directory"),
		                                         _("Create directory on server"))
		remote_menu.Append (self.idREMOTECHMOD, _("Change Permissions"),
		                                        _("Change permissions on serve"))
		remote_menu.Append (self.idREMOTEREMOVE, _("Remo&ve Directories"),
		                                         _("Remove files from server"))
		remote_menu.Append (self.idREMOTECHANGE, _("C&hange Directory"),
		                                         _("Change server directory"))

		# Set up the event handlers
		EVT_MENU (self, self.idREMOTEDOWNLOAD, self.onRemoteDownload)
		EVT_MENU (self, self.idREMOTERENAME, self.onRemoteRename)
		EVT_MENU (self, self.idREMOTEDELETE, self.onRemoteDelete)
		EVT_MENU (self, self.idREMOTECREATE, self.onRemoteCreate)
		EVT_MENU (self, self.idREMOTECHMOD, self.onRemoteChmod)
		EVT_MENU (self, self.idREMOTEREMOVE, self.onRemoteRemove)
		EVT_MENU (self, self.idREMOTECHANGE, self.onRemoteChange)

		return remote_menu

	def createHelpMenu (self):
		# Create the help menu
		help_menu = wxMenu ()
		help_menu.Append (self.idABOUT, _("&About"), _("About Ftpcube"))

		# Set up the event handlers
		EVT_MENU (self, self.idABOUT, self.onAbout)

		return help_menu

	def makeToolbar (self):
		# Create the toolbar
		self.toolbar = self.CreateToolBar ()
		self.toolbar.SetToolBitmapSize (wxSize (32, 32))

		bitmap = icons.connect.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_CONNECT, bitmap,
		                            shortHelpString=_("Normal Connect"))
		bitmap = icons.quick_connect.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_QUICKCONNECT, bitmap,
		                            shortHelpString=_("Quick Connect"))
		bitmap = icons.disconnect.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_DISCONNECT, bitmap,
		                            shortHelpString=_("Disconnect"))
		bitmap = icons.abort.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_ABORT, bitmap,
		                            shortHelpString=_("Abort Current Action"))
		bitmap = icons.refresh.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_REFRESH, bitmap,
		                            shortHelpString=_("Refresh"))
		util.AddLineSeparator (self.toolbar, 16)
		bitmap = icons.bookmark.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_BOOKMARK, bitmap,
		                            shortHelpString=_("Bookmark"))
		util.AddLineSeparator (self.toolbar, 16)
		bitmap = icons.exit.getBitmap ()
		self.toolbar.AddSimpleTool (self.idTOOL_QUIT, bitmap,
		                            shortHelpString=_("Quit"))

		self.dir_entry_box = wxComboBox (self.toolbar,
		                                 self.idTOOL_ENTRYBOX,
		                                 size=wxSize (350, -1),
		                                 style=wxCB_DROPDOWN)
		self.binary_button = wxRadioButton (self.toolbar, self.idTOOL_BINARY, _("BINARY"))
		self.ascii_button = wxRadioButton (self.toolbar, self.idTOOL_ASCII, _("ASCII"))
		if main.getApp ().getConfig ()['transfer_type'] == protocol.BINARY:
			self.binary_button.SetValue (true)
		else:
			self.ascii_button.SetValue (true)

		for i in range (5): self.toolbar.AddSeparator ()
		self.toolbar.AddControl (self.dir_entry_box)
		self.toolbar.AddSeparator ()
		self.toolbar.AddControl (self.binary_button)
		self.toolbar.AddSeparator ()
		self.toolbar.AddControl (self.ascii_button)

		# Set the event handlers
		EVT_TOOL (self.toolbar, self.idTOOL_CONNECT, self.onConnect)
		EVT_TOOL (self.toolbar, self.idTOOL_QUICKCONNECT, self.onQuickConnect)
		EVT_TOOL (self.toolbar, self.idTOOL_DISCONNECT, self.onDisconnect)
		EVT_TOOL (self.toolbar, self.idTOOL_ABORT, self.onAbort)
		EVT_TOOL (self.toolbar, self.idTOOL_REFRESH, self.onRefresh)
		EVT_TOOL (self.toolbar, self.idTOOL_BOOKMARK, self.onBookmarks)
		EVT_TOOL (self.toolbar, self.idTOOL_QUIT, self.onQuit)
		EVT_COMBOBOX (self.toolbar, self.idTOOL_ENTRYBOX, self.onComboSelect)
		EVT_RADIOBUTTON (self.binary_button, self.idTOOL_BINARY, self.onBinarySelect)
		EVT_RADIOBUTTON (self.ascii_button, self.idTOOL_ASCII, self.onAsciiSelect)

		self.toolbar.Realize ()
		return self.toolbar

	def getConsoleWindow (self):
		return self.consolewin

	def getControlWindow (self):
		return self.ctrlwin

	def getLocalWindow (self):
		return self.localwin

	def getRemoteWindow (self):
		return self.remotewin

	def onConnect (self, event):
		# Disconnect if necessary
		thread_mgr = main.getApp ().getThreadManager ()
		if thread_mgr.getMainThread ():
			if not self.promptDisconnect ():
				return
			self.onDisconnect (event)

		config_opts = main.getApp ().getConfig ().getOpts ()
		connect = connectwin.ConnectionWindow (self, opts=config_opts)
		ret = connect.ShowModal ()
		if ret == wxID_OK:
			connect_opts = connect.getOpts ()
			if connect_opts['host'] and connect_opts['port']:
				main.getApp ().initiateConnection (connect_opts)
			else:
				messages.displayErrorDialog (self,
					_("A hostname and port must be supplied."))

	def onQuickConnect (self, event):
		# Disconnect if necessary
		thread_mgr = main.getApp ().getThreadManager ()
		if thread_mgr.getMainThread ():
			if not self.promptDisconnect ():
				return
			self.onDisconnect (event)

		opts = { }
		opts.update (main.getApp ().getConfig ().getOpts ())
		host = messages.displayInputDialog (self,
			_("Ftpcube - Quick Connect"), _("Remote Host:"))
		if host:
			opts['host'] = host
			main.getApp ().initiateConnection (opts)

	def onBookmarks (self, event):
		bookmark = bookmarkwin.BookmarkWindow (self)
		bookmark.Show (true)

	def promptDisconnect (self):
		thread = main.getApp ().getThreadManager ().getMainThread ()
		ret = messages.displayMessageDialog (self, _("Ftpcube - Disconnect"),
			_("You are currently connected to: %(host)s\nDo you wish to disconnect and connect to a new host?")
			%{ 'host' : thread.getOpts ()['host'] },
			wxYES_NO)
		if ret == wxID_YES:
			return True
		return False

	def onDisconnect (self, event):
		thread_mgr = main.getApp ().getThreadManager ()
		main_thread = thread_mgr.getMainThread ()
		if main_thread:
			main_thread.setDone ()

		# Clear remote status stuff
		self.remotewin.clearCache ()
		self.remotewin.clearList ()
		self.dir_entry_box.Remove (0, self.dir_entry_box.GetLastPosition ())

	def onClearConsole (self, event):
		self.consolewin.onClear (event)

	def onOptions (self, event):
		config = main.getApp ().getConfig ()
		config_opts = config.getOpts ()
		options = optionwin.OptionWindow (self, opts=config_opts)
		ret = options.ShowModal ()
		if ret == wxID_OK:
			newopts = options.getOpts ()
			config_opts.update (newopts)
			config.saveConfig (config.getPath ())

	def onLocalUpload (self, event):
		self.localwin.onUpload (event)

	def onLocalRename (self, event):
		self.localwin.onRename (event)

	def onLocalDelete (self, event):
		self.localwin.onDelete (event)

	def onLocalCreate (self, event):
		self.localwin.onCreate (event)

	def onLocalChmod (self, event):
		self.localwin.onChmod (event)

	def onLocalChange (self, event):
		self.localwin.onChange (event)

	def onRemoteDownload (self, event):
		self.remotewin.onDownload (event)

	def onRemoteRename (self, event):
		self.remotewin.onRename (event)

	def onRemoteDelete (self, event):
		self.remotewin.onDelete (event)

	def onRemoteCreate (self, event):
		self.remotewin.onCreate (event)

	def onRemoteChmod (self, event):
		self.remotewin.onChmod (event)

	def onRemoteRemove (self, event):
		self.remotewin.onRemove (event)

	def onRemoteChange (self, event):
		self.remotewin.onChange (event)

	def onAbout (self, event):
		about = aboutwin.AboutWindow (self)
		ret = about.ShowModal ()

	def onAbort (self, event):
		main_thread = main.getApp ().getThreadManager ().getMainThread ()
		if main_thread:
			main_thread.abort ()

	def onRefresh (self, event):
		if main.getApp ().getThreadManager ().getMainThread ():
			self.remotewin.onRefresh (event)
		self.localwin.onRefresh (event)

	def onBinarySelect (self, event):
		if __debug__:
			print "Using binary mode for transfers."
		config = main.getApp ().getConfig ()
		config['transfer_type'] = protocol.BINARY

	def onAsciiSelect (self, event):
		if __debug__:
			print "Using ascii mode for transfers."
		config = main.getApp ().getConfig ()
		config['transfer_type'] = protocol.ASCII

	def onDirectoryChange (self, event):
		found = self.dir_entry_box.FindString (event.directory)
		if found == -1:
			if __debug__:
				print "Adding [%s] to main window combo box." %event.directory
			self.dir_entry_box.Append (event.directory)
			self.dir_entry_box.SetSelection (self.dir_entry_box.GetCount () - 1)
		else:
			if __debug__:
				print "Changing combo box directory selection: [%s]" %event.directory
			self.dir_entry_box.SetSelection (found)

	def onComboSelect (self, event):
		dir = self.dir_entry_box.GetString (event.GetSelection ())
		self.remotewin.changeDirectory (dir, self.remotewin.getDir ())

	def onQuit (self, event):
		config = main.getApp ().getConfig ()
		config['win_size'] = self.GetSizeTuple ()
		config.saveConfig (config.getPath ())
		self.Destroy ()
