#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

# Direction constants
DOWNLOAD = (1 << 0)
UPLOAD   = (1 << 1)

# Method constants
BINARY   = (1 << 2)
ASCII    = (1 << 3)

class ProtocolInterface:

	def __init__ (self, **kwargs):
		raise NoImplementedError

	def setPassive (self, passive):
		raise NotImplementedError

	def getPassive (self):
		raise NotImplementedError

	def getDispatcher (self):
		raise NotImplementedError

	def destroy (self):
		raise NotImplementedError

	def addOutgoing (self, cmd, handler):
		raise NotImplementedError

	def initiateConnect (self, handler=None):
		raise NotImplementedError

	def getWelcomeMsg (self, handler=None):
		raise NotImplementedError

	def login (self, username, password, handler=None):
		raise NotImplementedError

	def noop (self, handler=None):
		raise NotImplementedError

	def cwd (self, dir, handler=None):
		raise NotImplementedError

	def pwd (self, handler=None):
		raise NotImplementedError

	def delete (self, file, handler=None):
		raise NotImplementedError

	def rename (self, name, new_name, handler=None):
		raise NotImplementedError

	def mkdir (self, dir, handler=None):
		raise NotImplementedError

	def rmdir (self, dir, handler=None):
		raise NotImplementedError

	def quit (self, handler=None):
		raise NotImplementedError

	def abort (self, handler=None):
		raise NotImplementedError

	def list (self, args, status=None, handler=None):
		raise NotImplementedError

	def retrieveAscii (self, file, local_dir, status=None, passive=True, handler=None):
		raise NotImplementedError

	def uploadAscii (self, file, local_dir, status=None, passive=True, handler=None):
		raise NotImplementedError

	def retrieveBinary (self, file, local_dir, status=None, blocksize=8192, rest=None,
	                                           passive=True, handler=None):
		raise NotImplementedError

	def uploadBinary (self, file, local_dir, status=None, blocksize=8192, rest=None,
	                                         passive=True, handler=None):
		raise NotImplementedError
