#
# FtpCube
# Copyright (C) 2002 Michael Gilfix
#
# This file is part of FtpCube.
#
# You should have received a file COPYING containing license terms
# along with this program; if not, write to Michael Gilfix
# (mgilfix@eecs.tufts.edu) for a copy.
#
# This version of FtpCube is open source; you can redistribute it and/or
# modify it under the terms listed in the file COPYING.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import formatter
from wxPython.wx import *

class wxStaticWrapText (wxStaticText):

	def __init__ (self, *args, **kwargs):
		wxStaticText.__init__ (self, *args, **kwargs)
		try:
			self.align = kwargs['alignment']
		except KeyError:
			self.align = 'left'

	def SetLabel (self, string):
		text = self.formatText (string)
		wxStaticText.SetLabel (self, text)

	def formatText (self, text):
		maxwidth = self.getMaxTextWidth ()
		writer = StringWriter (maxwidth)
		writer.new_alignment (self.align)
		myformatter = formatter.AbstractFormatter (writer)
		for data in text.split ('\n'):
			myformatter.add_flowing_data (data)
			myformatter.end_paragraph (0)
		return writer.get_text ()

	def getMaxTextWidth (self):
		return 36

class StringWriter (formatter.NullWriter):

	HORIZONTAL_RULE_CHAR = '='

	def __init__ (self, maxwidth):
		formatter.NullWriter.__init__ (self)
		self.text = [ ]  # The text written so far
		self.line = [ ]  # The current line to operate on
		self.alignment = 'left'
		self.maxwidth = maxwidth
		self.horiz_char = self.HORIZONTAL_RULE_CHAR
		self.reset_pos ()

	def new_alignment (self, align):
		if align not in [ 'left', 'center', 'right' ]:
			raise ValueError, "invalid alignment: %s" %align
		self.alignment = align

	def get_horiz_char (self):
		return self.horiz_char

	def set_horiz_char (self, char):
		self.horiz_char = char

	def reset_pos (self, pos=0, word_break=False):
		self.pos = pos
		self.word_break = word_break

	def reset_line (self):
		self.line = [ ]
		self.reset_pos ()

	def send_paragraph (self, blankline):
		self.text.append (self.collapse_line ())
		self.text.extend ([ ' ' for i in range (blankline) ])
		self.reset_line ()

	def send_line_break (self):
		self.text.append (self.collapse_line ())
		self.reset_line ()

	def send_hor_rule (self, *args, **kwargs):
		self.text.append (self.collapse_line ())
		self.text.append (self.horiz_char * self.maxwidth)
		self.reset_line ()

	def send_flowing_data (self, data):
		if not data:
			return

		self.reset_pos (self.pos, self.word_break or data[0] in ' ')
		for word in data.split ():
			if self.word_break:
				if self.pos + len (word) >= self.maxwidth:
					self.text.append (self.collapse_line ())
					self.reset_line ()
				else:
					self.line.append (' ')
					self.reset_pos (self.pos + 1)

			self.line.append (word)
			self.reset_pos (self.pos + len (word), True)

		# Set the state of the final word
		self.reset_pos (self.pos, data[-1].isspace ())

	def send_literal_data (self, data):
		self.line.append (data)
		index = data.rfind ('\n')
		if index != -1:
			self.pos = 0
			data = data[index + 1: ]
		data.expandtabs (4)
		self.reset_pos (self.pos + len (data))

	def collapse_line (self):
		txt = ''.join (self.line)
		if self.alignment == 'left':
			txt = txt.ljust (self.maxwidth)
		elif self.alignment == 'center':
			txt = txt.center (self.maxwidth)
		else:
			txt = txt.rjust (self.maxwidth)
		return txt

	def get_text (self):
		return '\n'.join (self.text)
